"""
Copyright (C) 2021  Alexander "PapaTutuWawa"

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
import os
import json
import logging

logger = logging.getLogger("mira.storage.StorageManager")


class StorageManager:
    __instance = None

    @staticmethod
    def get_instance(file_location="/etc/mira/storage.json"):
        if not StorageManager.__instance:
            StorageManager(file_location=file_location)
        return StorageManager.__instance

    def __init__(self, file_location):
        if StorageManager.__instance:
            raise Exception("Trying to instanciate StorageManger twice")

        self._data = {}  # Module -> Section -> Data
        self._file_location = file_location

        logger.debug("Loading data from %s" % (file_location))
        if os.path.exists(file_location):
            with open(file_location, "r") as f:
                self._data = json.loads(f.read())

        StorageManager.__instance = self

    def get_data(self, module, section):
        """
        Get the data stored for module @module under the section
        @section. Returns {} if there is no data stored for @module.
        """
        if not module in self._data:
            logging.debug("get_data: module unknown in self._data")
            logging.debug('module: "%s"' % (module))
            return {}
        if not section in self._data[module]:
            logging.debug("get_data: section unknown in self._data[module]")
            logging.debug('module: "%s", section: "%s"' % (module, section))
            return {}

        return self._data[module][section]

    def set_data(self, module, section, data):
        """
        Stores the data @data for @module under section @section.
        Flushes the data to storage afterwards.
        """
        if not module in self._data:
            self._data[module] = {}
        self._data[module][section] = data
        self.__flush()

    def __flush(self):
        logger.debug("Flushing to storage")
        with open(self._file_location, "w") as f:
            f.write(json.dumps(self._data))
